# An application of our stable module methods to the example in the section
# "Example: Identification of a Stable Module" of the supporting appendix.

import numpy as np
import ODEToDigraph as od

# Define functional forms used in the update function
def H( x, k=0.5, n=2 ):
    xkn = ( x / k )**n
    return xkn / ( 1 + xkn )

# Define the following ordering of variables
u = 0
w = 1
x = 2
y = 3
z = 4

numVars = 5

# List of variable names coinciding with the above ordering
keys = ["u","w","x","y","z"]

# Define initial conditions; these do not affect stable module results
u0 = [0]*numVars
u0[u] = 0.5
u0[w] = 0.5
u0[x] = 0.5
u0[y] = 0.5
u0[z] = 0.5

def update( uu, t ):
    d = [0]*numVars 
    d[u] = uu[x]-uu[u]
    d[w] = H(uu[z])-uu[w]
    d[x] = H(uu[w])-uu[x]
    d[y] = H(uu[x])-uu[y]
    d[z] = 1+H(uu[y])*H(uu[u])-H(uu[u])-uu[z]
    
    return d 

maxVals = np.ones( numVars )
minVals = np.zeros( numVars )

tol = [0.001]*numVars

thresh, high, stable = od.stableModuleSearch( update, u0, minVals, maxVals, tol )
print()
print()
print("==========================================")
print()
for t,s,h in zip( thresh, stable, high ):
    print("-------------------------")
    od.printModuleNodes( t, h, keys = keys )
    print("-------------------------")
